function figure1c
% Generates Figure 1(c)
% from the paper "Deciding when to stop: Efficient experimentation to learn to predict drug-target interactions", Maja Temerinac-Ott, Armaghan W Naik and Robert F Murphy
%
% Author: Maja Temerinac-Ott
% Date: Jan 2015
% E-Mail: temerina@frias.uni-freiburg.de

% 07272105 icaoberg Updated code to be compatible with OSX/Linux

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Ion Channel
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%load the true interaction matrix
NrDataPath = [pwd, filesep, ...
    'PrimaryData' filesep 'ProteinDrugsDatabase' filesep ...
    'IonChannel' filesep];
load([NrDataPath,'Interactions.mat'],'Interactions');
load([NrDataPath,'ProteinSimilarity.mat']);
load([NrDataPath,'DrugSimilarity.mat']);

Y_true = Interactions;

% For all three methods, there should be the same number of random seeds
numSeeds = 5;
seed = [343,552,449,1606,32];
RandomRun = 1:5;

R = 20;
numRs = length(R);

percentageExperiments = cell(numSeeds,numRs);
evaluationMeasure = cell(numSeeds,numRs);
discoveryRate = cell(numSeeds,numRs);

percentageExperimentsRand = cell(numSeeds,numRs,length(RandomRun));
evaluationMeasureRand= cell(numSeeds,numRs,length(RandomRun));
discoveryRateRand = cell(numSeeds,numRs,length(RandomRun));

% If available use the intermediate data

IntermediateDataPathNr = [pwd, filesep,'IntermediateData' ...
    filesep 'IonChannel' filesep ];
if ~exist( IntermediateDataPathNr )
    mkdir( IntermediateDataPathNr );
end

for seed_idx = 1:numSeeds
    for R_idx = 1:numRs
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %1. Initialization using column of drugs:
        %(reaction of all proteins to one drug) + for each protein the
        %reaction to a random drug with active learning
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        IntermediateDataFile1 = [IntermediateDataPathNr,'Y_queried_for_each_run_col_initalization_R_',num2str(R(R_idx)),'_seed_',num2str(seed(seed_idx)),'.mat'];
        IntermediateDataFile2 = [IntermediateDataPathNr,'Y_predicted_for_each_run_col_initalization_R_',num2str(R(R_idx)),'_seed_',num2str(seed(seed_idx)),'.mat'];
        if(exist(IntermediateDataFile1,'file')&&exist(IntermediateDataFile2,'file'))
            load(IntermediateDataFile1);
            load(IntermediateDataFile2);
        else
            % Else do the experiments from scratch
            [Y_queired_all,Y_predict_all] = active_learning_KBMF_column(Interactions, DrugSimilarity,ProteinSimilarity,R(R_idx),seed(seed_idx));
            save(IntermediateDataFile1,'Y_queired_all')
            save(IntermediateDataFile2,'Y_predict_all')
        end
        percentageExperiments{seed_idx,R_idx} = computePercentageCompleted(Y_queired_all);
        % Compute the evaluation measure
        evaluationMeasure{seed_idx,R_idx} = computeEvaluationMeasure(Y_queired_all,Y_predict_all,Y_true);
        % Compute the discovery rate of interactions
        discoveryRate{seed_idx,R_idx} = computeDiscoveryRate(Y_queired_all,Y_true);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %2. Initialization using column of drugs:
        %(reaction of all proteins to one drug) + for each protein the
        %reaction to a random drug with random learning
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        for RandRun_idx = 1:length(RandomRun)
            IntermediateDataFile1 = [IntermediateDataPathNr,'Y_queried_for_each_run_col_initalization_R_',num2str(R(R_idx)),'_seed_',num2str(seed(seed_idx)),'_RandomRunSeed_',num2str(RandomRun(RandRun_idx)),'.mat'];
            IntermediateDataFile2 = [IntermediateDataPathNr,'Y_predicted_for_each_run_col_initalization_R_',num2str(R(R_idx)),'_seed_',num2str(seed(seed_idx)),'_RandomRunSeed_',num2str(RandomRun(RandRun_idx)),'.mat'];
            
            if(exist(IntermediateDataFile1,'file')&&exist(IntermediateDataFile2,'file'))
                load(IntermediateDataFile1);
                load(IntermediateDataFile2);
            else
                % Else do the experiments from scratch
                [Y_queired_all,Y_predict_all] = random_learning_KBMF_column(Interactions, DrugSimilarity,ProteinSimilarity,R(R_idx),seed(seed_idx),RandomRun(RandRun_idx));
                save(IntermediateDataFile1,'Y_queired_all')
                save(IntermediateDataFile2,'Y_predict_all')
            end
            percentageExperimentsRand{seed_idx,R_idx,RandRun_idx} = computePercentageCompleted(Y_queired_all);
            % Compute the evaluation measure
            evaluationMeasureRand{seed_idx,R_idx,RandRun_idx} = computeEvaluationMeasure(Y_queired_all,Y_predict_all,Y_true);
            % Compute the discovery rate of interactions
            discoveryRateRand{seed_idx,R_idx,RandRun_idx} = computeDiscoveryRate(Y_queired_all,Y_true);
        end
    end
end

%Plot the figures
R_idx=1;

%Compute mean and standard deviation of the accuracy
percent_of_experiments=percentageExperiments{1,1};
percent_of_experiments_rand=percentageExperimentsRand{1,1,1};

Accuracy = zeros(length(percent_of_experiments),numSeeds);
AccuracyRand = zeros(length(percent_of_experiments_rand),numSeeds*length(RandomRun));

count = 1;
for seed_idx = 1:numSeeds
    Accuracy(:,seed_idx) = evaluationMeasure{seed_idx,R_idx,1};
    for RandRun_idx = 1:length(RandomRun)
        AccuracyRand(:,count) = evaluationMeasureRand{seed_idx,R_idx,RandRun_idx};
        count = count+1;
    end
end
M = mean(Accuracy');
S = std(Accuracy');
M_rand = mean(AccuracyRand');
S_rand = std(AccuracyRand');

try
    figure;
    hold on
    plot(percent_of_experiments,M,'k')
    plot(percent_of_experiments,M+S,':k')
    plot(percent_of_experiments,M-S,':k')
    plot(percent_of_experiments_rand,M_rand,'g')
    plot(percent_of_experiments_rand,M_rand+S_rand,':g')
    plot(percent_of_experiments_rand,M_rand-S_rand,':g')
    
    xlabel('Percent of Experiments for Training')
    ylabel('Accuracy on Remaining Experiments')
    title(['R = ',num2str(R(R_idx)),' Ion Channel'])
    axis([0 1 0.88 1])
    hold off
    saveas(gcf, 'figure1c.jpg' )
    close
catch err
    warning( 'Unable to open figure. Skipping Figura 1c.' )
    getReport( err )
end
